/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#include "camMenu.h"

CamMenu::CamMenu()
{
	imgSize = cv::Size(320, 240);
	camMenuImg = cv::Mat(imgSize.height + _MENUBOTTOMHEIGHT, imgSize.width + _MENULEFTWIDTH, CV_8UC3, cv::Scalar(0, 0, 0));
	menutreegui.setMenuDim(imgSize.width + _MENULEFTWIDTH, imgSize.height + _MENUBOTTOMHEIGHT, _MENULEFTWIDTH, _MENUBOTTOMHEIGHT);
	camImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(155));
	numCamera = 0;								/** Numero de camara a abrir */
	key = 0;
	salir = false;
	espacioDeColor = "RGB";
	modoMedicion = false;
	modoDistorsion = false;
	ajusteLuz = 0;
	tamObjeto.clear();
	text.clear();
	initMenu();
}

void CamMenu::initMenu()
{
	menutreegui.addMenu("Informacion", "Use los cursores/intro para navegar");
	menutreegui.addMenu("Resolucion", "Cambia la resolucion de la imagen");
		menutreegui.addSubMenu("160x120", "Cambia la resolucion a 160x120 pixeles");
		menutreegui.addSubMenu("320x240", "Cambia la resolucion a 320x240 pixeles");
		menutreegui.addSubMenu("640x480", "Cambia la resolucion a 640x480 pixeles");
		menutreegui.addSubMenu("Medir objeto", "Activa regla para medir objetos con el raton");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Espacio color", "Cambia el espacio de color");
		menutreegui.addSubMenu("RGB", "Espacio de color activo");
		menutreegui.addSubMenu("R/G/B", "Alterna los canales (R)/G/B");
		menutreegui.addSubMenu("HSV", "Cambiar a espacio de color HSV");
		menutreegui.addSubMenu("H/S/V", "Alterna los canales (H)/S/V");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Distorsion", "Muestra la distorsion de lente en la imagen");
	menutreegui.addMenu("Ajuste de luz", "Muestra el efecto de la luz en la camara");
		menutreegui.addSubMenu("Aumentar luz", "Aumenta la luminosidad de la imagen");
		menutreegui.addSubMenu("Disminuir luz", "Disminuye la luminosidad de la imagen");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
}

int CamMenu::openCamera(int ncam)
{
	cv::namedWindow("Camera", CV_WINDOW_AUTOSIZE);/** Define ventana opencv para mostrar camara */
	numCamera = ncam;
	if ( !streamCapture.open(numCamera))
		return (0);
	streamCapture >> frame;
	if (frame.empty() )
		return (0);
	else
		cv::resize( frame, camImg , camImg.size(), 0, 0, cv::INTER_LINEAR );
	cv::Mat tmpImg = cv::Mat(camMenuImg, cv::Rect( _MENULEFTWIDTH, 0, imgSize.width, imgSize.height) );
	camImg.copyTo(tmpImg);
	menutreegui.drawLeftMenu(camMenuImg);
	menutreegui.drawBottomMenu(camMenuImg, text);	
	cv::imshow("Camera", camMenuImg);
	key = cv::waitKey(50);
	return (1);
}

int CamMenu::update(int wait)
{
	streamCapture >> frame;
	if ( frame.empty() )
		camImg.setTo(155);
	else
		cv::resize( frame, camImg , camImg.size(), 0, 0, cv::INTER_LINEAR );
	applyOptionsToImage();
	readAnExeUserInput();
	menutreegui.drawLeftMenu(camMenuImg);
	menutreegui.drawBottomMenu(camMenuImg, text);	
	setBottomMenuMessage();
	cv::imshow("Camera", camMenuImg);
	key = cv::waitKey(wait);
	return (key);
}

int CamMenu::Exit() { return (salir); }

void CamMenu::applyOptionsToImage()
{
	if ( espacioDeColor.compare("HSV") == 0 )
		cv::cvtColor( camImg, camImg, CV_BGR2HSV );
	else if ( espacioDeColor.compare("H") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::cvtColor( camImg, camImg, CV_BGR2HSV );
		cv::split( camImg, splitted );
		splitted[1] = splitted[0]; splitted[2] = splitted[0];
		cv::merge( splitted, camImg);
	}
	else if ( espacioDeColor.compare("S") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::cvtColor( camImg, camImg, CV_BGR2HSV );
		cv::split( camImg, splitted );
		splitted[0] = splitted[1]; splitted[2] = splitted[1];
		cv::merge( splitted, camImg);
	}
	else if ( espacioDeColor.compare("V") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::cvtColor( camImg, camImg, CV_BGR2HSV );
		cv::split( camImg, splitted );
		splitted[0] = splitted[2]; splitted[1] = splitted[2];
		cv::merge( splitted, camImg);
	}
	else if ( espacioDeColor.compare("B") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::split( camImg, splitted );
		splitted[1] = splitted[0]; splitted[2] = splitted[0];
		cv::merge( splitted, camImg);
		//camImg -= cv::Scalar(0, 255,255);
	}
	else if ( espacioDeColor.compare("G") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::split( camImg, splitted );
		splitted[0] = splitted[1]; splitted[2] = splitted[1];
		cv::merge( splitted, camImg);
		//camImg -= cv::Scalar(255, 0,255);
	}
	else if ( espacioDeColor.compare("R") == 0 )
	{
		std::vector<cv::Mat> splitted;
		cv::split( camImg, splitted );
		splitted[0] = splitted[2]; splitted[1] = splitted[2];
		cv::merge( splitted, camImg);
		//camImg -= cv::Scalar(255, 255,0);
	}
	if (ajusteLuz != 0)
	{
		if (ajusteLuz > 0)
			camImg += cv::Scalar(ajusteLuz, ajusteLuz, ajusteLuz);
		else
			camImg -= cv::Scalar(-ajusteLuz, -ajusteLuz, -ajusteLuz);
	}
	cv::Mat tmpImg = cv::Mat(camMenuImg, cv::Rect( _MENULEFTWIDTH, 0, imgSize.width, imgSize.height) );
	camImg.copyTo(tmpImg);
	if (modoMedicion)
	{
		if (tamObjeto.size() <= 1)
			cv::line(camMenuImg, cv::Point(tamObjeto[0].x,0), cv::Point(tamObjeto[0].x,imgSize.height), cv::Scalar(0,0,255), 1);
		else
		{
			cv::line(camMenuImg, cv::Point(tamObjeto[0].x,0), cv::Point(tamObjeto[0].x,imgSize.height), cv::Scalar(0,0,255), 1);
			cv::line(camMenuImg, cv::Point(tamObjeto[1].x,0), cv::Point(tamObjeto[1].x,imgSize.height), cv::Scalar(0,0,255), 1);
		}
	}
	if (modoDistorsion)
	{
		cv::rectangle(camMenuImg, cv::Point(_MENULEFTWIDTH+5,5), cv::Point(imgSize.width + _MENULEFTWIDTH - 5, imgSize.height - 5), cv::Scalar(0,0,255), 1, 8, 0);
	}
}

void CamMenu::readAnExeUserInput()
{
	if ( key == 82)
		menutreegui.previousMenu();
	else if ( key == 84)
		menutreegui.nextMenu();
	else if ( key == 27 )
	{
		menutreegui.exitSubmenu();
		modoMedicion = false;
		modoDistorsion = false;
		ajusteLuz = 0;
	}
	else if ( key == 10)
	{
		std::string mSelected = menutreegui.getHighlightedMenu();
		if ( menutreegui.highlightedIndexHasSubmenu() )
			menutreegui.enterSubmenu();
		else if ( mSelected.compare("Volver") == 0 )
		{
			menutreegui.exitSubmenu();
			modoMedicion = false;
			modoDistorsion = false;
			ajusteLuz = 0;
		}
		else if ( mSelected.compare("Salir") == 0 )
			salir = true;
		else if ( mSelected.compare("160x120") == 0 || mSelected.compare("320x240") == 0 || mSelected.compare("640x480") == 0)
		{
			if ( mSelected.compare("160x120") == 0)
				imgSize = cv::Size(160, 120);
			if ( mSelected.compare("320x240") == 0)
				imgSize = cv::Size(320, 240);
			if ( mSelected.compare("640x480") == 0)
				imgSize = cv::Size(640, 480);
			camMenuImg = cv::Mat(imgSize.height + _MENUBOTTOMHEIGHT, imgSize.width + _MENULEFTWIDTH, CV_8UC3, cv::Scalar(0, 0, 0));
			menutreegui.setMenuDim(imgSize.width + _MENULEFTWIDTH, imgSize.height + _MENUBOTTOMHEIGHT, _MENULEFTWIDTH, _MENUBOTTOMHEIGHT);
			camImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(0,0,0));
		}
		else if ( mSelected.compare("Medir objeto") == 0 )
		{
			modoMedicion = !modoMedicion;
			if (modoMedicion)
				tamObjeto.push_back(cv::Point(_MENULEFTWIDTH+1, 0));
			else
				tamObjeto.clear();
		}
		else if ( mSelected.compare("RGB") == 0 )
			espacioDeColor = "RGB";
		else if ( mSelected.compare("R/G/B") == 0 )
		{
			std::string info = menutreegui.getHighlightedMenuInfo();
			if (info.find("(R)") != std::string::npos)
			{
				espacioDeColor = "R";
				menutreegui.setHighlightedMenuInfo("Alterna los canales R/(G)/B");
			}
			if (info.find("(G)") != std::string::npos)
			{
				espacioDeColor = "G";
				menutreegui.setHighlightedMenuInfo("Alterna los canales R/G/(B)");
			}
			if (info.find("(B)") != std::string::npos)
			{
				espacioDeColor = "B";
				menutreegui.setHighlightedMenuInfo("Alterna los canales (R)/G/B");
			}
		}
		else if ( mSelected.compare("HSV") == 0 )
			espacioDeColor = "HSV";
		else if ( mSelected.compare("H/S/V") == 0 )
		{
			std::string info = menutreegui.getHighlightedMenuInfo();
			if (info.find("(H)") != std::string::npos)
			{
				espacioDeColor = "H";
				menutreegui.setHighlightedMenuInfo("Alterna los canales H/(S)/V");
			}
			if (info.find("(S)") != std::string::npos)
			{
				espacioDeColor = "S";
				menutreegui.setHighlightedMenuInfo("Alterna los canales H/S/(V)");
			}
			if (info.find("(V)") != std::string::npos)
			{
				espacioDeColor = "V";
				menutreegui.setHighlightedMenuInfo("Alterna los canales (H)/S/V");
			}
		}
		else if ( mSelected.compare("Distorsion") == 0 )
			modoDistorsion = !modoDistorsion;
		else if ( mSelected.compare("Aumentar luz") == 0 )
			ajusteLuz = (ajusteLuz <= 250)?(ajusteLuz+5):ajusteLuz;
		else if ( mSelected.compare("Disminuir luz") == 0 )
			ajusteLuz = (ajusteLuz >= -250)?(ajusteLuz-5):ajusteLuz;
		else
			menutreegui.exitSubmenu();
	}
	if (key == 81 && modoMedicion)
	{
		if (tamObjeto.size() <= 1)
			tamObjeto[0].x = (tamObjeto[0].x > _MENULEFTWIDTH+1)?(tamObjeto[0].x-1):(_MENULEFTWIDTH+1);
		else
			tamObjeto[1].x = (tamObjeto[1].x > _MENULEFTWIDTH+1)?(tamObjeto[1].x-1):(_MENULEFTWIDTH+1);
	}
	if (key == 83 && modoMedicion)
	{
		if (tamObjeto.size() <= 1)
			tamObjeto[0].x = (tamObjeto[0].x < (imgSize.width+_MENULEFTWIDTH))?(tamObjeto[0].x+1):(imgSize.width+_MENULEFTWIDTH);
		else
			tamObjeto[1].x = (tamObjeto[1].x < (imgSize.width+_MENULEFTWIDTH))?(tamObjeto[1].x+1):(imgSize.width+_MENULEFTWIDTH);
	}
	if (key == 32 && modoMedicion)
	{
		if (tamObjeto.size() <= 1)
			tamObjeto.push_back( cv::Point( tamObjeto[0].x+5,0 ) );
	}
}

void CamMenu::setBottomMenuMessage()
{	/** Establece el mensaje a mostrar al usuario en el menu inferior */
	std::string mSelected = menutreegui.getHighlightedMenu();
	text.clear();
	if ( mSelected.compare("Informacion") == 0 )
	{
		text.push_back( menutreegui.getHighlightedMenuInfo() );
		if ( streamCapture.isOpened() )
			text.push_back( cv::format("Abierta camara: %d", numCamera) );
		else
			text.push_back( cv::format("Error abriendo camara: %d", numCamera) );
	}
	else if (modoMedicion)
	{
		text.push_back("Pulse cursor derecha/izquierda para mover las reglas");
		if (tamObjeto.size() <= 1)
			text.push_back("Pulse espacio para fijar las reglas");
		else
			text.push_back( cv::format("Ancho del objeto: %d", tamObjeto[1].x - tamObjeto[0].x) );
	}
	else
	{
		text.push_back( mSelected + ":");
		text.push_back( menutreegui.getHighlightedMenuInfo() );
	}
}
